#ifndef lint
static char rcsid[] = "$Id: map_file.c,v 1.1 1995/08/08 14:17:03 mleisher Exp $";
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "map_file.h"


#if HAVE_GETPAGESIZE
extern int getpagesize();
#else
#define getpagesize() 4096
#endif

#ifdef HP
/*int munmap(caddr_t addr, size_t len);*/
extern int close(int);
#else
#ifdef __STDC__
extern int munmap(char *, int);
extern int close(int);
#else
extern int munmap();
extern int close();
#endif
#endif


size_t
#ifdef __STDC__
file_size(char *filename)
#else
file_size(filename)
char *filename;
#endif
{
    struct stat s;

    if (stat(filename, &s) < 0) {
        fprintf(stderr, "map_file: cannot stat %s\n", filename);
        perror("stat");
        return -1;
    }
    return (size_t) s.st_size;
}

/* code which when given a file name will return a pointer to memory
   which contains a read-only copy of the contents of the file.  this
   is done with virtual memory tricks, not by reading the file */

char *
#ifdef __STDC__
map_file(char *filename, size_t len, size_t off)
#else
map_file(filename, len, off)
char *filename;
size_t len, off;
#endif
{
    int fd;
    char *r;
    size_t k;

    /*
     * Avoid a warning about the rcsid not being used when compiling
     * with all warning flags on.
     */
    r = rcsid;

    if ((fd = open(filename, O_RDONLY, 0)) < 0) {
	fprintf(stderr, "map_file: cannot open %s\n", filename);
	return NULL;
    }

    if (len == 0)
      len = file_size(filename);

    /*
     * Adjust the file length to page boundries for better efficiency.
     */
    k = (size_t) getpagesize();
    len = (len+k-1)/k;
    len = len*k;

    /*
     * Adjust the offset to page boundries.
     */
    off = (off+k-1)/k;
    off = off*k;

    r = mmap(NULL, len, PROT_READ, MAP_SHARED, fd, off);
    if (r == (char *) -1) {
	fprintf(stderr, "map_file: cannot map file %s\n", filename);
	perror("map_file");
    }

    close(fd);

    return r;
}

int
#ifdef __STDC__
unmap_file(char *data, size_t len)
#else
unmap_file(data, len)
char *data;
size_t len;
#endif
{
    if (munmap(data, len) < 0) {
        fprintf(stderr, "map_file: can't unmap file data.\n");
        perror("munmap");
        return -1;
    }
    return 0;
}
